@@ -14,7 +14,7 @@ TKR <- sample_ohlc_data
 cloud <- ichimoku(TKR)
 
 ## ----strat--------------------------------------------------------------------
-strat <- strat(cloud, c1 = "close", c2 = "tenkan")
+strat <- strat(cloud, c1 = "cloudB", c2 = "kijun")
 print(strat[100:105], plot = FALSE)
 
 ## ----summary------------------------------------------------------------------
@@ -24,7 +24,7 @@ summary(strat)
 plot(strat)
 
 ## ----combine------------------------------------------------------------------
-strat2 <- strat(cloud, "cloudBase", "kijun")
+strat2 <- strat(cloud, "kijun", "tenkan")
 
 newstrat <- stratcombine(strat, strat2)
 summary(newstrat)
@@ -32,7 +32,7 @@ summary(newstrat)
 ## ----perfana, eval=FALSE------------------------------------------------------
 #  library(PerformanceAnalytics)
 #  # To chart performance comparison of strategy vs benchmark, daily returns and drawdowns
-#  charts.PerformanceSummary(strat[ ,c("sret", "ret")])
+#  charts.PerformanceSummary(strat[, c("sret", "ret")])
 #  # For a table detailing drawdowns, including depth, length of recovery etc.
 #  table.Drawdowns(strat[, "sret"])
 
@@ -40,10 +40,10 @@ summary(newstrat)
 autostrat(cloud, n = 3)
 
 ## ----autostrat2---------------------------------------------------------------
-autostrat(cloud, n = 2, dir = "short", level = "2")
+autostrat(cloud, n = 3, dir = "short", level = "2")
 
 ## ----autostrat3---------------------------------------------------------------
-autostrat(cloud, n = 2, dir = "long", level = "3")
+autostrat(cloud, n = 3, dir = "long", level = "3")
 
 ## ----mlgrid-------------------------------------------------------------------
 mlgrid(cloud, y = "logret", dir = "long", type = "boolean", unique = TRUE)[100:105, 1:4]
@@ -51,8 +51,3 @@ mlgrid(cloud, y = "logret", dir = "long", type = "boolean", unique = TRUE)[100:1
 ## ----mlgrid2------------------------------------------------------------------
 mlgrid(cloud, y = "ret", dir = "short", type = "numeric", unique = FALSE)[100:105, 1:4]
 
-## ----look---------------------------------------------------------------------
-look(cloud)
-
-look(strat)
-
